<?php

namespace Drupal\markitdown\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\File\FileSystemInterface;
use Drupal\file\FileInterface;
use Drupal\markitdown\Service\MarkdownConverter;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for MarkItDown module.
 */
class MarkItDownController extends ControllerBase {

  /**
   * The markdown converter service.
   *
   * @var \Drupal\markitdown\Service\MarkdownConverter
   */
  protected $markdownConverter;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * Constructs a new MarkItDownController object.
   *
   * @param \Drupal\markitdown\Service\MarkdownConverter $markdown_converter
   *   The markdown converter service.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   */
  public function __construct(
    MarkdownConverter $markdown_converter,
    FileSystemInterface $file_system
  ) {
    $this->markdownConverter = $markdown_converter;
    $this->fileSystem = $file_system;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('markitdown.converter'),
      $container->get('file_system')
    );
  }

  /**
   * Convert a file to markdown.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file to convert.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response containing the converted content or status.
   */
  public function convertFile(FileInterface $file, Request $request) {
    if (!$file) {
      throw new NotFoundHttpException();
    }

    $file_uri = $file->getFileUri();
    $real_path = $this->fileSystem->realpath($file_uri);

    if (!$real_path) {
      throw new NotFoundHttpException();
    }

    // Get output mode from request parameters.
    $output_mode = $request->query->get('output_mode', 'text');
    $output_path = $request->query->get('output_path', NULL);

    // 处理输出路径中可能包含的特殊字符
    if (!empty($output_path)) {
      // 确保输出路径是UTF-8编码
      if (!mb_check_encoding($output_path, 'UTF-8')) {
        $output_path = mb_convert_encoding($output_path, 'UTF-8');
      }

      // 处理Windows路径中的反斜杠
      $output_path = str_replace('\\', '/', $output_path);

      // 如果输出路径不是绝对路径，则转换为绝对路径
      if (!$this->fileSystem->realpath($output_path) && strpos($output_path, '/') !== 0) {
        // 使用站点默认文件目录作为基础路径
        $output_path = 'public://' . $output_path;
      }
    }

    // Convert the file based on output mode.
    if ($output_mode === 'file' && !empty($output_path)) {
      // File mode - save to specified path.
      $result = $this->markdownConverter->convertToMarkdown($real_path, [
        'output_mode' => 'file',
        'output_path' => $output_path,
      ]);

      if (!$result) {
        $this->messenger()->addError($this->t('Failed to convert file to Markdown.'));
        return $this->redirect('entity.file.canonical', ['file' => $file->id()]);
      }

      $this->messenger()->addStatus($this->t('File successfully converted to Markdown and saved to @path.', [
        '@path' => $output_path,
      ]));

      return $this->redirect('entity.file.canonical', ['file' => $file->id()]);
    }
    else {
      // Text mode - return the content.
      $markdown_content = $this->markdownConverter->convertToMarkdown($real_path, [
        'output_mode' => 'text',
      ]);

      if ($markdown_content === FALSE) {
        $this->messenger()->addError($this->t('Failed to convert file to Markdown.'));
        return $this->redirect('entity.file.canonical', ['file' => $file->id()]);
      }

      // Return the markdown content as a response.
      $response = new Response($markdown_content);
      $response->headers->set('Content-Type', 'text/markdown');

      return $response;
    }
  }

}
