<?php

namespace Drupal\markitdown\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure MarkItDown settings.
 */
class MarkItDownSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'markitdown.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'markitdown_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('markitdown.settings');

    // 转换器类型设置
    $form['converter_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Converter Settings'),
      '#open' => TRUE,
    ];

    $form['converter_settings']['default_converter_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Default Converter Type'),
      '#description' => $this->t('Select which converter to use by default.'),
      '#options' => [
        'command_line' => $this->t('Command Line Tool (local execution)'),
        'api' => $this->t('API Service (remote execution)'),
      ],
      '#default_value' => $config->get('default_converter_type') ?: 'command_line',
    ];

    // 命令行工具设置
    $form['command_line_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Command Line Tool Settings'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="default_converter_type"]' => ['value' => 'command_line'],
        ],
      ],
    ];

    $form['command_line_settings']['markitdown_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Markitdown Path'),
      '#description' => $this->t('The path to the markitdown executable. Leave empty to use the system default (if markitdown is in your PATH).'),
      '#default_value' => $config->get('markitdown_path'),
    ];

    // API 设置
    $form['api_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('API Service Settings'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          ':input[name="default_converter_type"]' => ['value' => 'api'],
        ],
      ],
    ];

    $form['api_settings']['api_endpoint'] = [
      '#type' => 'url',
      '#title' => $this->t('API Endpoint'),
      '#description' => $this->t('The URL of the Markdown conversion API service.'),
      '#default_value' => $config->get('api_endpoint'),
      '#required' => FALSE,
    ];

    $form['api_settings']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('API key for authentication with the service (if required).'),
      '#default_value' => $config->get('api_key'),
      '#required' => FALSE,
    ];

    $form['api_settings']['api_format'] = [
      '#type' => 'select',
      '#title' => $this->t('API Format'),
      '#description' => $this->t('The format to use when sending files to the API.'),
      '#options' => [
        'multipart' => $this->t('Multipart Form (standard file upload)'),
        'base64' => $this->t('Base64 JSON (file encoded in JSON request)'),
        'binary' => $this->t('Binary (raw file content in request body)'),
      ],
      '#default_value' => $config->get('api_format') ?: 'multipart',
    ];

    $form['api_settings']['api_format_description'] = [
      '#type' => 'markup',
      '#markup' => '<div class="description">' . $this->t('
        <strong>Multipart Form:</strong> Standard HTTP file upload using multipart/form-data, most widely supported.<br>
        <strong>Base64 JSON:</strong> File content is Base64 encoded and sent as part of a JSON object, useful for REST APIs.<br>
        <strong>Binary:</strong> Raw file content is sent directly in the request body, with appropriate Content-Type header.
      ') . '</div>',
    ];

    // 输出设置
    $form['output_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Output Settings'),
      '#open' => TRUE,
    ];

    $form['output_settings']['default_output_mode'] = [
      '#type' => 'radios',
      '#title' => $this->t('Default Output Mode'),
      '#description' => $this->t('The default output mode to use when converting files.'),
      '#options' => [
        'text' => $this->t('Text (return content directly)'),
        'file' => $this->t('File (save to specified path)'),
      ],
      '#default_value' => $config->get('default_output_mode') ?: 'text',
    ];

    $form['output_settings']['default_output_directory'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Default Output Directory'),
      '#description' => $this->t('The default directory where converted files will be saved when using file output mode.'),
      '#default_value' => $config->get('default_output_directory'),
      '#states' => [
        'visible' => [
          ':input[name="default_output_mode"]' => ['value' => 'file'],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('markitdown.settings')
      ->set('default_converter_type', $form_state->getValue('default_converter_type'))
      ->set('markitdown_path', $form_state->getValue('markitdown_path'))
      ->set('api_endpoint', $form_state->getValue('api_endpoint'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('api_format', $form_state->getValue('api_format'))
      ->set('default_output_mode', $form_state->getValue('default_output_mode'))
      ->set('default_output_directory', $form_state->getValue('default_output_directory'))
      ->save();

    parent::submitForm($form, $form_state);
  }

}
