<?php

namespace Drupal\markitdown\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;
use Symfony\Component\Mime\MimeTypes;

/**
 * API implementation of the converter strategy.
 */
class ApiStrategy implements ConverterStrategyInterface {

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Constructs a new ApiStrategy object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    FileSystemInterface $file_system,
    ConfigFactoryInterface $config_factory,
    ClientInterface $http_client
  ) {
    $this->logger = $logger_factory->get('markitdown');
    $this->fileSystem = $file_system;
    $this->configFactory = $config_factory;
    $this->httpClient = $http_client;
  }

  /**
   * {@inheritdoc}
   */
  public function convert($file_path, array $options = []) {
    // 记录转换请求开始
    $request_id = isset($options['request_id']) ? $options['request_id'] : uniqid('api_');
    $this->logger->info('开始通过API转换文件 [@req_id]: @file', [
      '@req_id' => $request_id,
      '@file' => $file_path,
    ]);

    // Set default options.
    $options += [
      'output_mode' => 'text',
      'output_path' => NULL,
    ];

    // 确保文件路径使用正确的编码
    $file_path = $this->sanitizePath($file_path);

    if (!file_exists($file_path)) {
      $this->logger->error('文件未找到 [@req_id]: @file', [
        '@req_id' => $request_id,
        '@file' => $file_path,
      ]);
      return FALSE;
    }

    // 读取文件内容与基本信息
    $file_content = file_get_contents($file_path);
    if ($file_content === FALSE) {
      $this->logger->error('无法读取文件内容 [@req_id]: @file', [
        '@req_id' => $request_id,
        '@file' => $file_path,
      ]);
      return FALSE;
    }

    // Get API endpoint configuration.
    $config = $this->configFactory->get('markitdown.settings');
    $api_endpoint = $config->get('api_endpoint');
    $api_key = $config->get('api_key');

    if (empty($api_endpoint)) {
      $this->logger->error('API端点未配置，无法进行转换 [@req_id]', ['@req_id' => $request_id]);
      return FALSE;
    }

    try {
      // Detect file mime type
      $mime_types = new MimeTypes();
      $mime_type = $mime_types->guessMimeType($file_path);

      // 准备文件上传请求
      $options_request = $this->prepareApiRequest($file_path, basename($file_path), $mime_type, $api_key, $options);

      // Make the API request
      $response = $this->httpClient->request('POST', $api_endpoint, $options_request);

      // 获取响应状态和大小
      $status_code = $response->getStatusCode();
      $response_body = (string) $response->getBody();

      // 处理JSON响应并提取markdown内容
      $markdown_content = $this->processApiResponse($response_body, $request_id);
      if ($markdown_content === FALSE) {
        $this->logger->error('从API响应中提取Markdown内容失败 [@req_id]', ['@req_id' => $request_id]);
        return FALSE;
      }

      // Save or return the content based on the output mode
      if ($options['output_mode'] === 'file' && !empty($options['output_path'])) {
        $output_path = $this->sanitizePath($options['output_path']);

        // Ensure output directory exists
        $output_dir = dirname($output_path);
        if (!file_exists($output_dir)) {
          $this->fileSystem->mkdir($output_dir, NULL, TRUE);
        }

        // Write the content to the output file
        $bytes_written = file_put_contents($output_path, $markdown_content);
        if ($bytes_written !== FALSE) {
          $this->logger->info('成功保存转换结果到文件 [@req_id]: @file', [
            '@req_id' => $request_id,
            '@file' => $output_path,
          ]);
          return TRUE;
        }
        else {
          $this->logger->error('写入输出文件失败 [@req_id]: @file', [
            '@req_id' => $request_id,
            '@file' => $output_path,
          ]);
          return FALSE;
        }
      }
      else {
        return $markdown_content;
      }
    }
    catch (RequestException $e) {
      // 详细记录API请求错误
      $error_message = $e->getMessage();
      $response = $e->getResponse();

      if ($response) {
        $status_code = $response->getStatusCode();
        $response_body = (string) $response->getBody();
        $this->logger->error('API请求失败 [@req_id]: 状态码=@code, 错误=@error, 响应=@response', [
          '@req_id' => $request_id,
          '@code' => $status_code,
          '@error' => $error_message,
          '@response' => $response_body,
        ]);
      }
      else {
        $this->logger->error('API请求失败，无响应 [@req_id]: @error', [
          '@req_id' => $request_id,
          '@error' => $error_message,
        ]);
      }

      return FALSE;
    }
    catch (\Exception $e) {
      $this->logger->error('API转换过程中发生错误 [@req_id]: @error', [
        '@req_id' => $request_id,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * 处理API响应，从JSON中提取markdown内容。
   *
   * @param string $response_body
   *   API响应内容。
   * @param string $request_id
   *   请求ID，用于日志记录。
   *
   * @return string|false
   *   成功返回提取的markdown内容，失败返回FALSE。
   */
  protected function processApiResponse($response_body, $request_id) {
    try {
      // 尝试解析JSON响应
      $json_data = json_decode($response_body, TRUE);

      // 检查JSON解析是否成功
      if (json_last_error() !== JSON_ERROR_NONE) {
        $error = json_last_error_msg();
        $this->logger->error('JSON解析失败 [@req_id]: @error', [
          '@req_id' => $request_id,
          '@error' => $error,
        ]);

        // 如果不是有效的JSON但内容不为空，可能是直接返回的markdown文本
        if (!empty(trim($response_body))) {
          return $response_body;
        }

        return FALSE;
      }

      // 检查是否包含markdown字段
      if (isset($json_data['markdown'])) {
        return $json_data['markdown'];
      }

      // 检查是否包含content字段（备选字段）
      if (isset($json_data['content'])) {
        return $json_data['content'];
      }

      // 检查响应状态码
      if (isset($json_data['code'])) {
        $code = (string) $json_data['code'];

        if ($code === '200' || $code === 200) {
          // 如果有detail字段，尝试作为内容返回
          if (isset($json_data['detail']) && is_string($json_data['detail'])) {
            return $json_data['detail'];
          }
        }
        else {
          // 非正常响应码
          $error_msg = isset($json_data['detail']) ? json_encode($json_data['detail']) : '未知错误';
          $this->logger->error('API响应状态码异常 [@req_id]: @code, 错误: @error', [
            '@req_id' => $request_id,
            '@code' => $code,
            '@error' => $error_msg,
          ]);
          return FALSE;
        }
      }

      // 如果JSON格式正确但结构不符合预期
      $this->logger->warning('API响应格式不符合预期 [@req_id]: 未找到markdown字段或状态码', [
        '@req_id' => $request_id,
      ]);

      // 最后尝试使用原始响应作为内容
      return $response_body;
    }
    catch (\Exception $e) {
      $this->logger->error('处理API响应时发生异常 [@req_id]: @error', [
        '@req_id' => $request_id,
        '@error' => $e->getMessage(),
      ]);
      return FALSE;
    }
  }

  /**
   * 准备API请求选项，根据不同的API类型构建不同的请求结构。
   *
   * @param string $file_path
   *   文件路径。
   * @param string $file_name
   *   文件名称。
   * @param string $mime_type
   *   文件MIME类型。
   * @param string $api_key
   *   API密钥。
   * @param array $options
   *   转换选项。
   *
   * @return array
   *   请求选项数组。
   */
  protected function prepareApiRequest($file_path, $file_name, $mime_type, $api_key, array $options) {
    $config = $this->configFactory->get('markitdown.settings');
    $api_format = $config->get('api_format') ?: 'multipart';

    $request_options = [];

    // 添加通用请求头
    $headers = [];
    if (!empty($api_key)) {
      $headers['X-API-Key'] = $api_key;
    }

    // 根据不同API格式设置请求选项
    switch ($api_format) {
      case 'base64':
        // 使用Base64编码文件内容并以JSON格式发送
        $file_content = base64_encode(file_get_contents($file_path));

        $request_options = [
          'headers' => array_merge($headers, ['Content-Type' => 'application/json']),
          'json' => [
            'fileName' => $file_name,
            'fileType' => $mime_type,
            'fileContent' => $file_content,
            'options' => $options,
          ],
        ];
        break;

      case 'binary':
        // 发送原始二进制文件内容
        $file_content = file_get_contents($file_path);

        $request_options = [
          'headers' => array_merge($headers, [
            'Content-Type' => $mime_type,
            'Content-Disposition' => 'attachment; filename="' . $file_name . '"',
          ]),
          'body' => $file_content,
        ];
        break;

      case 'multipart':
      default:
        // 默认使用multipart/form-data方式上传文件
        $multipart = [
          [
            'name' => 'file',
            'contents' => fopen($file_path, 'r'),
            'filename' => $file_name,
            'headers' => [
              'Content-Type' => $mime_type,
            ],
          ],
        ];

        // 添加额外的表单字段
        foreach ($options as $key => $value) {
          if (is_string($value) || is_numeric($value)) {
            $multipart[] = [
              'name' => $key,
              'contents' => (string) $value,
            ];
          }
        }

        $request_options = [
          'headers' => $headers,
          'multipart' => $multipart,
        ];
        break;
    }

    return $request_options;
  }

  /**
   * Sanitize a file path to handle special characters.
   *
   * @param string $path
   *   The file path to sanitize.
   *
   * @return string
   *   The sanitized file path.
   */
  protected function sanitizePath($path) {
    // 确保路径使用UTF-8编码
    if (!mb_check_encoding($path, 'UTF-8')) {
      $path = mb_convert_encoding($path, 'UTF-8');
    }

    // 处理Windows路径中的反斜杠
    $path = str_replace('\\', '/', $path);

    // 确保路径是绝对路径
    if (!$this->fileSystem->realpath($path)) {
      // 如果不是绝对路径，尝试获取绝对路径
      $realpath = $this->fileSystem->realpath($path);
      if ($realpath) {
        $path = $realpath;
      }
    }

    return $path;
  }

}
