<?php

namespace Drupal\markitdown\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Command line implementation of the converter strategy.
 */
class CommandLineStrategy implements ConverterStrategyInterface {

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The path to the markitdown executable.
   *
   * @var string
   */
  protected $markitdownPath;

  /**
   * Constructs a new CommandLineStrategy object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    FileSystemInterface $file_system,
    ConfigFactoryInterface $config_factory
  ) {
    $this->logger = $logger_factory->get('markitdown');
    $this->fileSystem = $file_system;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public function convert($file_path, array $options = []) {
    // 记录转换请求开始
    $this->logger->info('开始通过命令行工具转换文件: @file', ['@file' => $file_path]);

    // Set default options.
    $options += [
      'output_mode' => 'text',
      'output_path' => NULL,
    ];

    // 确保文件路径使用正确的编码
    $file_path = $this->sanitizePath($file_path);

    if (!file_exists($file_path)) {
      $this->logger->error('文件未找到: @file', ['@file' => $file_path]);
      return FALSE;
    }

    // 记录文件信息
    $file_size = filesize($file_path);
    $this->logger->info('文件信息: 路径=@path, 大小=@size 字节, 扩展名=@ext', [
      '@path' => $file_path,
      '@size' => $file_size,
      '@ext' => pathinfo($file_path, PATHINFO_EXTENSION),
    ]);

    // Check if markitdown is installed.
    $markitdown_path = $this->getMarkitdownPath();
    $this->logger->info('使用markitdown工具路径: @path', ['@path' => $markitdown_path]);

    if (!$this->isMarkitdownInstalled()) {
      $this->logger->error('markitdown工具未安装或未在路径中找到: @path', ['@path' => $markitdown_path]);
      return FALSE;
    }

    $this->logger->info('markitdown工具已安装，准备执行转换');

    // Prepare the command.
    if ($options['output_mode'] === 'file' && !empty($options['output_path'])) {
      // 确保输出路径使用正确的编码
      $output_path = $this->sanitizePath($options['output_path']);
      $this->logger->info('准备将转换结果保存到文件: @file', ['@file' => $output_path]);

      // 确保输出目录存在
      $output_dir = dirname($output_path);
      if (!file_exists($output_dir)) {
        $this->logger->debug('创建输出目录: @dir', ['@dir' => $output_dir]);
        $this->fileSystem->mkdir($output_dir, NULL, TRUE);
      }

      // File mode - save to specified path.
      $command = sprintf('%s %s -o %s',
        escapeshellcmd($markitdown_path),
        escapeshellarg($file_path),
        escapeshellarg($output_path)
      );

      $this->logger->info('以文件输出模式执行命令');
      $result = $this->executeCommand($command);

      if ($result === FALSE) {
        $this->logger->error('命令执行失败，无法将文件转换为Markdown: @file', ['@file' => $file_path]);
      } else {
        $this->logger->info('命令执行成功，已保存转换结果到: @file', ['@file' => $output_path]);
      }

      return $result;
    }
    else {
      // Text mode - return the content.
      $command = sprintf('%s %s',
        escapeshellcmd($markitdown_path),
        escapeshellarg($file_path)
      );

      $this->logger->info('以文本输出模式执行命令');
      $output = $this->executeCommand($command);
      if ($output === FALSE) {
        $this->logger->error('命令执行失败，无法将文件转换为Markdown: @file', ['@file' => $file_path]);
        return FALSE;
      }

      $output_size = strlen($output);
      $this->logger->info('命令执行成功，获取到 @size 字节的Markdown内容', ['@size' => $output_size]);

      // 对于大型响应，记录一部分内容作为示例
      if ($output_size > 1000) {
        $preview = substr($output, 0, 200) . '... [截断]';
        $this->logger->debug('转换内容预览: @preview', ['@preview' => $preview]);
      }

      return $output;
    }
  }

  /**
   * Execute a shell command.
   *
   * @param string $command
   *   The command to execute.
   *
   * @return mixed
   *   The output of the command or TRUE/FALSE for success/failure.
   */
  protected function executeCommand($command) {
    // 记录开始执行时间
    $start_time = microtime(TRUE);

    // 设置正确的环境变量以处理UTF-8字符
    $locale = setlocale(LC_CTYPE, 0);
    setlocale(LC_CTYPE, 'en_US.UTF-8');
    $this->logger->debug('设置执行环境: LC_CTYPE=en_US.UTF-8');

    // 设置环境变量以确保正确处理Unicode字符
    $old_lang = getenv('LANG');
    putenv('LANG=en_US.UTF-8');

    // 获取系统环境变量PATH
    $path = getenv('PATH');
    $bin_env_path = dirname($this->getMarkitdownPath());

    // 添加自定义路径到PATH
    putenv("PATH=$path:$bin_env_path");
    $this->logger->debug('已将markitdown所在目录添加到PATH: @path', ['@path' => $bin_env_path]);

    // 记录执行的命令
    $this->logger->info('执行命令: @command', ['@command' => $command]);

    // 使用system函数执行命令
    $return_var = 0;
    ob_start();
    system($command . ' 2>&1', $return_var);
    $output = ob_get_clean();

    // 计算执行耗时
    $execution_time = round((microtime(TRUE) - $start_time) * 1000);

    // 恢复原始环境设置
    setlocale(LC_CTYPE, $locale);
    if ($old_lang !== false) {
      putenv("LANG=$old_lang");
    }
    else {
      putenv('LANG');
    }
    $this->logger->debug('已恢复原始环境设置');

    if ($return_var !== 0) {
      $this->logger->error('命令执行失败: @command, 返回值: @code, 耗时: @time 毫秒', [
        '@command' => $command,
        '@code' => $return_var,
        '@time' => $execution_time,
      ]);

      if (!empty($output)) {
        $this->logger->error('命令输出: @output', ['@output' => $output]);
      }

      return FALSE;
    }

    $output_size = strlen($output);
    $this->logger->info('命令执行成功，返回值: @code, 输出大小: @size 字节, 耗时: @time 毫秒', [
      '@code' => $return_var,
      '@size' => $output_size,
      '@time' => $execution_time,
    ]);

    return $output;
  }

  /**
   * Check if markitdown tool is installed.
   *
   * @return bool
   *   TRUE if markitdown is installed, FALSE otherwise.
   */
  protected function isMarkitdownInstalled() {
    // 确保路径使用正确的编码
    $markitdown_path = $this->getMarkitdownPath();
    $bin_env_path = dirname($markitdown_path);
    $command = sprintf('%s --version', escapeshellcmd($markitdown_path));
    $this->logger->debug('检查markitdown是否已安装: @command', ['@command' => $command]);

    // 设置正确的环境变量以处理UTF-8字符
    $locale = setlocale(LC_CTYPE, 0);
    setlocale(LC_CTYPE, 'en_US.UTF-8');

    // 设置环境变量以确保正确处理Unicode字符
    $old_lang = getenv('LANG');
    putenv('LANG=en_US.UTF-8');

    // 获取系统环境变量PATH
    $path = getenv('PATH');
    // 添加自定义路径到PATH
    putenv("PATH=$path:$bin_env_path");

    // 使用system函数执行命令
    $return_var = 0;
    ob_start();
    system($command . ' 2>&1', $return_var);
    $output = ob_get_clean();

    // 恢复原始环境设置
    setlocale(LC_CTYPE, $locale);
    if ($old_lang !== false) {
      putenv("LANG=$old_lang");
    }
    else {
      putenv('LANG');
    }

    if ($return_var === 0) {
      $this->logger->debug('markitdown工具已安装，版本信息: @version', [
        '@version' => trim($output),
      ]);
      return TRUE;
    }
    else {
      $this->logger->warning('markitdown工具未安装或版本检查失败: @output', [
        '@output' => $output,
      ]);
      return FALSE;
    }
  }

  /**
   * Get markitdown executable path.
   *
   * @return string
   *   The path to the markitdown executable.
   */
  protected function getMarkitdownPath() {
    if ($this->markitdownPath) {
      return $this->markitdownPath;
    }

    $config = $this->configFactory->get('markitdown.settings');
    if ($config->get('markitdown_path')) {
      $markitdown_path = $config->get('markitdown_path');
      $this->logger->debug('从配置中获取markitdown路径: @path', ['@path' => $markitdown_path]);
    }
    else {
      $markitdown_path = trim(shell_exec('which markitdown'));
      if (empty($markitdown_path)) {
        $markitdown_path = 'markitdown';
        $this->logger->debug('未找到markitdown路径，使用默认名称: markitdown');
      }
      else {
        $this->logger->debug('通过which命令找到markitdown路径: @path', ['@path' => $markitdown_path]);
      }
    }

    $this->markitdownPath = $this->sanitizePath($markitdown_path);
    return $this->markitdownPath;
  }

  /**
   * Sanitize a file path to handle special characters.
   *
   * @param string $path
   *   The file path to sanitize.
   *
   * @return string
   *   The sanitized file path.
   */
  protected function sanitizePath($path) {
    // 确保路径使用UTF-8编码
    if (!mb_check_encoding($path, 'UTF-8')) {
      $path = mb_convert_encoding($path, 'UTF-8');
      $this->logger->debug('路径编码已转换为UTF-8: @path', ['@path' => $path]);
    }

    // 处理Windows路径中的反斜杠
    $original_path = $path;
    $path = str_replace('\\', '/', $path);
    if ($original_path !== $path) {
      $this->logger->debug('Windows路径已标准化: @original -> @new', [
        '@original' => $original_path,
        '@new' => $path,
      ]);
    }

    // 确保路径是绝对路径
    if (!$this->fileSystem->realpath($path)) {
      // 如果不是绝对路径，尝试获取绝对路径
      $realpath = $this->fileSystem->realpath($path);
      if ($realpath) {
        $original_path = $path;
        $path = $realpath;
        $this->logger->debug('相对路径已转换为绝对路径: @original -> @absolute', [
          '@original' => $original_path,
          '@absolute' => $path,
        ]);
      }
    }

    return $path;
  }

}
