<?php

namespace Drupal\markitdown\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;

/**
 * Service for converting various file formats to Markdown using markitdown tool.
 */
class MarkdownConverter {

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The command line strategy.
   *
   * @var \Drupal\markitdown\Service\CommandLineStrategy
   */
  protected $commandLineStrategy;

  /**
   * The API strategy.
   *
   * @var \Drupal\markitdown\Service\ApiStrategy
   */
  protected $apiStrategy;

  /**
   * Constructs a new MarkdownConverter object.
   *
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\markitdown\Service\CommandLineStrategy $command_line_strategy
   *   The command line strategy.
   * @param \Drupal\markitdown\Service\ApiStrategy $api_strategy
   *   The API strategy.
   */
  public function __construct(
    LoggerChannelFactoryInterface $logger_factory,
    FileSystemInterface $file_system,
    ConfigFactoryInterface $config_factory,
    CommandLineStrategy $command_line_strategy,
    ApiStrategy $api_strategy
  ) {
    $this->logger = $logger_factory->get('markitdown');
    $this->fileSystem = $file_system;
    $this->configFactory = $config_factory;
    $this->commandLineStrategy = $command_line_strategy;
    $this->apiStrategy = $api_strategy;
  }

  /**
   * Convert a file to Markdown format.
   *
   * @param string $file_path
   *   The path to the file to convert.
   * @param array $options
   *   An array of options:
   *   - output_mode: Either 'text' to return the content or 'file' to save it.
   *   - output_path: The path where to save the file (only for 'file' mode).
   *   - converter_type: The converter type to use ('command_line' or 'api').
   *     If not provided, the default from configuration will be used.
   *
   * @return mixed
   *   If output_mode is 'text', returns the markdown content as a string.
   *   If output_mode is 'file', returns TRUE on success, FALSE otherwise.
   */
  public function convertToMarkdown($file_path, array $options = []) {
    // 开始记录转换请求
    $start_time = microtime(TRUE);
    $request_id = uniqid('markitdown_');
    $this->logger->info('收到Markdown转换请求 [@req_id]: 文件=@file', [
      '@req_id' => $request_id,
      '@file' => $file_path,
    ]);

    // Set default options.
    $options += [
      'output_mode' => 'text',
      'output_path' => NULL,
      'converter_type' => NULL,
    ];

    // 记录转换选项
    $this->logger->info('转换选项 [@req_id]: @options', [
      '@req_id' => $request_id,
      '@options' => json_encode($options),
    ]);

    // Get the converter type from options or fallback to configuration.
    $converter_type = $options['converter_type'];
    if (empty($converter_type)) {
      $config = $this->configFactory->get('markitdown.settings');
      $converter_type = $config->get('default_converter_type') ?: 'command_line';
      $this->logger->info('使用配置的默认转换器类型 [@req_id]: @type', [
        '@req_id' => $request_id,
        '@type' => $converter_type,
      ]);
    }
    else {
      $this->logger->info('使用请求指定的转换器类型 [@req_id]: @type', [
        '@req_id' => $request_id,
        '@type' => $converter_type,
      ]);
    }

    // Sanitize the file path.
    $original_path = $file_path;
    $file_path = $this->sanitizePath($file_path);
    if ($original_path !== $file_path) {
      $this->logger->debug('路径已标准化 [@req_id]: @orig -> @new', [
        '@req_id' => $request_id,
        '@orig' => $original_path,
        '@new' => $file_path,
      ]);
    }

    // Check if file exists.
    if (!file_exists($file_path)) {
      $this->logger->error('文件未找到 [@req_id]: @file', [
        '@req_id' => $request_id,
        '@file' => $file_path,
      ]);
      return FALSE;
    }

    // Get the appropriate converter strategy.
    $strategy = $this->getConverterStrategy($converter_type);
    $strategy_class = get_class($strategy);
    $this->logger->info('选择转换策略 [@req_id]: @strategy', [
      '@req_id' => $request_id,
      '@strategy' => $strategy_class,
    ]);

    try {
      // 增加请求ID到选项中，便于追踪
      $options['request_id'] = $request_id;

      // Convert the file using the selected strategy.
      $this->logger->info('开始执行转换处理 [@req_id]', ['@req_id' => $request_id]);
      $result = $strategy->convert($file_path, $options);

      // 计算总处理时间
      $total_time = round((microtime(TRUE) - $start_time) * 1000);

      if ($result === FALSE) {
        $this->logger->error('转换失败 [@req_id]: 总耗时 @time 毫秒', [
          '@req_id' => $request_id,
          '@time' => $total_time,
        ]);
      }
      else {
        $this->logger->info('转换成功 [@req_id]: 总耗时 @time 毫秒', [
          '@req_id' => $request_id,
          '@time' => $total_time,
        ]);

        if ($options['output_mode'] === 'text') {
          $result_size = is_string($result) ? strlen($result) : 0;
          $this->logger->info('返回文本结果 [@req_id]: 大小 @size 字节', [
            '@req_id' => $request_id,
            '@size' => $result_size,
          ]);
        }
      }

      return $result;
    }
    catch (\Exception $e) {
      $this->logger->error('转换过程中发生异常 [@req_id]: @error', [
        '@req_id' => $request_id,
        '@error' => $e->getMessage(),
      ]);

      // 记录异常堆栈
      $this->logger->debug('异常堆栈 [@req_id]: @trace', [
        '@req_id' => $request_id,
        '@trace' => $e->getTraceAsString(),
      ]);

      return FALSE;
    }
  }

  /**
   * Get the appropriate converter strategy based on the type.
   *
   * @param string $converter_type
   *   The converter type ('command_line' or 'api').
   *
   * @return \Drupal\markitdown\Service\ConverterStrategyInterface
   *   The converter strategy.
   */
  protected function getConverterStrategy($converter_type) {
    switch ($converter_type) {
      case 'api':
        $this->logger->debug('使用API转换策略');
        return $this->apiStrategy;

      case 'command_line':
      default:
        $this->logger->debug('使用命令行转换策略');
        return $this->commandLineStrategy;
    }
  }

  /**
   * Sanitize a file path to handle special characters.
   *
   * @param string $path
   *   The file path to sanitize.
   *
   * @return string
   *   The sanitized file path.
   */
  protected function sanitizePath($path) {
    // 确保路径使用UTF-8编码
    if (!mb_check_encoding($path, 'UTF-8')) {
      $path = mb_convert_encoding($path, 'UTF-8');
      $this->logger->debug('路径编码已转换为UTF-8: @path', ['@path' => $path]);
    }

    // 处理Windows路径中的反斜杠
    $original_path = $path;
    $path = str_replace('\\', '/', $path);
    if ($original_path !== $path) {
      $this->logger->debug('Windows路径已标准化: @original -> @new', [
        '@original' => $original_path,
        '@new' => $path,
      ]);
    }

    // 确保路径是绝对路径
    if (!$this->fileSystem->realpath($path)) {
      // 如果不是绝对路径，尝试获取绝对路径
      $realpath = $this->fileSystem->realpath($path);
      if ($realpath) {
        $original_path = $path;
        $path = $realpath;
        $this->logger->debug('相对路径已转换为绝对路径: @original -> @absolute', [
          '@original' => $original_path,
          '@absolute' => $path,
        ]);
      }
    }

    return $path;
  }

}
